package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus;

package .tmp;

import com.google.common.base.Preconditions;
import com.mojang.datafixers.kinds.App;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.util.Function5;
import com.mojang.serialization.Codec;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import dev.shadowsoffire.apotheosis.adventure.affix.effect.PotionAffix;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.placebo.codec.PlaceboCodecs;
import java.util.Map;
import net.minecraft.ChatFormatting;
import net.minecraft.core.BlockPos;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.util.StringUtil;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.effect.MobEffect;
import net.minecraft.world.effect.MobEffectInstance;
import net.minecraft.world.effect.MobEffectUtil;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.entity.projectile.AbstractArrow;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.EntityHitResult;
import net.minecraft.world.phys.HitResult;
import net.minecraftforge.registries.ForgeRegistries;

public class PotionBonus extends GemBonus {
  public static final Codec<dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.PotionBonus> CODEC;
  
  protected final MobEffect effect;
  
  protected final PotionAffix.Target target;
  
  protected final Map<LootRarity, EffectData> values;
  
  protected final boolean stackOnReapply;
  
  static {
    CODEC = RecordCodecBuilder.create(inst -> inst.group(gemClass(), (App)ForgeRegistries.MOB_EFFECTS.getCodec().fieldOf("mob_effect").forGetter(()), (App)PotionAffix.Target.CODEC.fieldOf("target").forGetter(()), (App)LootRarity.mapCodec(EffectData.CODEC).fieldOf("values").forGetter(()), (App)PlaceboCodecs.nullableField((Codec)Codec.BOOL, "stack_on_reapply", Boolean.valueOf(false)).forGetter(())).apply((Applicative)inst, dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.PotionBonus::new));
  }
  
  public PotionBonus(GemClass gemClass, MobEffect effect, PotionAffix.Target target, Map<LootRarity, EffectData> values, boolean stackOnReapply) {
    super(Apotheosis.loc("mob_effect"), gemClass);
    this.effect = effect;
    this.target = target;
    this.values = values;
    this.stackOnReapply = stackOnReapply;
  }
  
  public Component getSocketBonusTooltip(ItemStack gem, LootRarity rarity) {
    MobEffectInstance inst = ((EffectData)this.values.get(rarity)).build(this.effect);
    MutableComponent comp = this.target.toComponent(new Object[] { toComponent(inst) }).m_130940_(ChatFormatting.YELLOW);
    int cooldown = getCooldown(rarity);
    if (cooldown != 0) {
      MutableComponent mutableComponent = Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(cooldown) });
      comp = comp.m_130946_(" ").m_7220_((Component)mutableComponent);
    } 
    if (this.stackOnReapply)
      comp = comp.m_130946_(" ").m_7220_((Component)Component.m_237115_("affix.apotheosis.stacking")); 
    return (Component)comp;
  }
  
  public int getNumberOfUUIDs() {
    return 0;
  }
  
  public Codec<? extends GemBonus> getCodec() {
    return (Codec)CODEC;
  }
  
  public boolean supports(LootRarity rarity) {
    return this.values.containsKey(rarity);
  }
  
  public void doPostHurt(ItemStack gem, LootRarity rarity, LivingEntity user, Entity attacker) {
    if (this.target == PotionAffix.Target.HURT_SELF) {
      applyEffect(gem, user, rarity);
    } else if (this.target == PotionAffix.Target.HURT_ATTACKER && 
      attacker instanceof LivingEntity) {
      LivingEntity tLiving = (LivingEntity)attacker;
      applyEffect(gem, tLiving, rarity);
    } 
  }
  
  public void doPostAttack(ItemStack gem, LootRarity rarity, LivingEntity user, Entity target) {
    if (this.target == PotionAffix.Target.ATTACK_SELF) {
      applyEffect(gem, user, rarity);
    } else if (this.target == PotionAffix.Target.ATTACK_TARGET && 
      target instanceof LivingEntity) {
      LivingEntity tLiving = (LivingEntity)target;
      applyEffect(gem, tLiving, rarity);
    } 
  }
  
  public void onBlockBreak(ItemStack gem, LootRarity rarity, Player player, LevelAccessor world, BlockPos pos, BlockState state) {
    if (this.target == PotionAffix.Target.BREAK_SELF)
      applyEffect(gem, (LivingEntity)player, rarity); 
  }
  
  public void onArrowImpact(ItemStack gemStack, LootRarity rarity, AbstractArrow arrow, HitResult res) {
    if (this.target == PotionAffix.Target.ARROW_SELF) {
      Entity entity = arrow.m_19749_();
      if (entity instanceof LivingEntity) {
        LivingEntity owner = (LivingEntity)entity;
        applyEffect(gemStack, owner, rarity);
      } 
    } else if (this.target == PotionAffix.Target.ARROW_TARGET && 
      res.m_6662_() == HitResult.Type.ENTITY) {
      Entity entity = ((EntityHitResult)res).m_82443_();
      if (entity instanceof LivingEntity) {
        LivingEntity target = (LivingEntity)entity;
        applyEffect(gemStack, target, rarity);
      } 
    } 
  }
  
  public float onShieldBlock(ItemStack gem, LootRarity rarity, LivingEntity entity, DamageSource source, float amount) {
    if (this.target == PotionAffix.Target.BLOCK_SELF) {
      applyEffect(gem, entity, rarity);
    } else if (this.target == PotionAffix.Target.BLOCK_ATTACKER) {
      Entity entity1 = source.m_7640_();
      if (entity1 instanceof LivingEntity) {
        LivingEntity target = (LivingEntity)entity1;
        applyEffect(gem, target, rarity);
      } 
    } 
    return amount;
  }
  
  protected int getCooldown(LootRarity rarity) {
    EffectData data = this.values.get(rarity);
    return data.cooldown;
  }
  
  private void applyEffect(ItemStack gemStack, LivingEntity target, LootRarity rarity) {
    int cooldown = getCooldown(rarity);
    if (cooldown != 0 && Affix.isOnCooldown(getCooldownId(gemStack), cooldown, target))
      return; 
    EffectData data = this.values.get(rarity);
    MobEffectInstance inst = target.m_21124_(this.effect);
    if (this.stackOnReapply && inst != null) {
      if (inst != null) {
        MobEffectInstance newInst = new MobEffectInstance(this.effect, Math.max(inst.m_19557_(), data.duration), inst.m_19564_() + 1 + data.amplifier);
        target.m_7292_(newInst);
      } 
    } else {
      target.m_7292_(data.build(this.effect));
    } 
    Affix.startCooldown(getCooldownId(gemStack), target);
  }
  
  public static Component toComponent(MobEffectInstance inst) {
    MutableComponent mutablecomponent = Component.m_237115_(inst.m_19576_());
    MobEffect mobeffect = inst.m_19544_();
    if (inst.m_19564_() > 0)
      mutablecomponent = Component.m_237110_("potion.withAmplifier", new Object[] { mutablecomponent, Component.m_237115_("potion.potency." + inst.m_19564_()) }); 
    if (inst.m_19557_() > 20)
      mutablecomponent = Component.m_237110_("potion.withDuration", new Object[] { mutablecomponent, MobEffectUtil.m_267641_(inst, 1.0F) }); 
    return (Component)mutablecomponent.m_130940_(mobeffect.m_19483_().m_19497_());
  }
  
  public dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.PotionBonus validate() {
    Preconditions.checkNotNull(this.effect, "Null mob effect");
    Preconditions.checkNotNull(this.target, "Null target");
    Preconditions.checkNotNull(this.values, "Null values map");
    return this;
  }
}
